@@ -1,5 +1,4 @@
-# coding: utf-8
-u"""
+"""
 Perform straight-line Wilson plot fit.  Draw the Wilson plot.
 
 Reflection d-spacings are determined from the crystal symmetry (from
@@ -35,15 +34,13 @@ Examples:
 
 """
 
-from __future__ import absolute_import, division, print_function
-
 import logging
 import time
+from io import StringIO
 from typing import Iterable, List, Optional, Sequence, Union
 
 import numpy as np
 from scipy.optimize import curve_fit
-from six.moves import cStringIO as StringIO
 from tabulate import tabulate
 
 import iotbx.phil
@@ -54,7 +51,7 @@ from dials.array_family import flex
 from dials.util import log
 from dials.util.options import OptionParser
 from dials.util.version import dials_version
-from dxtbx.model import Experiment, ExperimentList
+from dxtbx.model import ExperimentList
 
 from screen19 import __version__, d_ticks, plot_intensities, terminal_size
 
@@ -66,7 +63,7 @@ help_message = __doc__
 
 
 phil_scope = iotbx.phil.parse(
-    u"""
+    """
     verbosity = 0
         .type = int(value_min=0)
         .caption = 'Verbosity level of log output'
@@ -120,9 +117,8 @@ logger = logging.getLogger(logger_name)
 debug, info, warn = logger.debug, logger.info, logger.warning
 
 
-def scaled_debye_waller(x, b, a):
-    # type: (float, float, float) -> float
-    u"""
+def scaled_debye_waller(x: float, b: float, a: float) -> float:
+    """
     Calculate a scaled isotropic Debye-Waller factor.
 
     By assuming a single isotropic disorder parameter, :param:`b`, this factor
@@ -140,9 +136,13 @@ def scaled_debye_waller(x, b, a):
     return a * np.exp(-b / 2 * x)
 
 
-def wilson_fit(d_star_sq, intensity, sigma, wilson_fit_max_d):
-    # type: (FloatSequence, FloatSequence, FloatSequence, float) -> Fit
-    u"""
+def wilson_fit(
+    d_star_sq: FloatSequence,
+    intensity: FloatSequence,
+    sigma: FloatSequence,
+    wilson_fit_max_d: float,
+) -> Fit:
+    """
     Fit a simple Debye-Waller factor, assume isotropic disorder parameter.
 
     Reflections with d ≥ :param:`wilson_fit_max_d` are ignored.
@@ -161,7 +161,7 @@ def wilson_fit(d_star_sq, intensity, sigma, wilson_fit_max_d):
 
     """
     # Eliminate reflections with d > wilson_fit_max_d from the fit
-    sel = d_star_sq > 1 / wilson_fit_max_d ** 2
+    sel = d_star_sq > 1 / wilson_fit_max_d**2
 
     # Perform a weighted Wilson plot fit to the reflection intensities
     fit, cov = curve_fit(
@@ -175,9 +175,10 @@ def wilson_fit(d_star_sq, intensity, sigma, wilson_fit_max_d):
     return fit
 
 
-def wilson_plot_ascii(miller_array, d_ticks=None):
-    # type: (miller.array, Optional[Sequence]) -> None
-    u"""
+def wilson_plot_ascii(
+    miller_array: miller.array, d_ticks: Optional[Sequence] = None
+) -> None:
+    """
     Print an ASCII-art Wilson plot of reflection intensities.
 
     Equivalent reflections will be merged according to the crystal symmetry.
@@ -196,7 +197,7 @@ def wilson_plot_ascii(miller_array, d_ticks=None):
     binned_intensity = [x if x else 0 for x in wilson.data[1:-1]]
     bins = dict(zip(wilson.binner.bin_centers(1), binned_intensity))
     if d_ticks:
-        tick_positions = ", ".join(['"%g" %s' % (d, 1 / d ** 2) for d in d_ticks])
+        tick_positions = ", ".join([f'"{d:g}" {1 / d ** 2}' for d in d_ticks])
         tick_positions = tick_positions.join(["(", ")"])
     else:
         tick_positions = ""
@@ -213,15 +214,14 @@ def wilson_plot_ascii(miller_array, d_ticks=None):
 
 
 def wilson_plot_image(
-    d_star_sq,  # type: FloatSequence
-    intensity,  # type: FloatSequence
-    fit,  # type: Fit
-    max_d=None,  # type: Optional[float]
-    ticks=None,  # type: Optional[FloatSequence]
-    output="wilson_plot",  # type: str
-):
-    # type: (...) -> None
-    u"""
+    d_star_sq: FloatSequence,
+    intensity: FloatSequence,
+    fit: Fit,
+    max_d: Optional[float] = None,
+    ticks: Optional[FloatSequence] = None,
+    output: str = "wilson_plot",
+) -> None:
+    """
     Generate the Wilson plot as a PNG image.
 
     :param:`max_d` allows greying out of the reflections not included in the
@@ -241,10 +241,10 @@ def wilson_plot_image(
     matplotlib.use("Agg")
     from matplotlib import pyplot as plt
 
-    plt.xlabel(u"d (Å) (inverse-square scale)")
-    plt.ylabel(u"Intensity (counts)")
+    plt.xlabel("d (Å) (inverse-square scale)")
+    plt.ylabel("Intensity (counts)")
     if ticks:
-        plt.xticks([1 / d ** 2 for d in ticks], ["%g" % d for d in ticks])
+        plt.xticks([1 / d**2 for d in ticks], [f"{d:g}" for d in ticks])
 
     # Matplotlib v3.3.0 includes API change 'nonposy' → 'nonpositive'
     # https://matplotlib.org/api/api_changes.html#log-symlog-scale-base-ticks-and-nonpos-specification
@@ -271,9 +271,10 @@ def wilson_plot_image(
     plt.close()
 
 
-def suggest_minimum_exposure(expts, refls, params):
-    # type: (ExperimentList[Experiment], flex.reflection_table, scope_extract) -> None
-    u"""
+def suggest_minimum_exposure(
+    expts: ExperimentList, refls: flex.reflection_table, params: scope_extract
+) -> None:
+    """
     Suggest an estimated minimum sufficient exposure to achieve a certain resolution.
 
     The estimate is based on a fit of a Debye-Waller factor under the assumption that a
@@ -349,8 +350,8 @@ def suggest_minimum_exposure(expts, refls, params):
 
     # Get recommended exposure factors
     # Use the fact that σ² = I for indexed data, so I/σ = √̅I
-    desired_d_star_sq = [1 / d ** 2 for d in desired_d]
-    target_i = min_i_over_sigma ** 2
+    desired_d_star_sq = [1 / d**2 for d in desired_d]
+    target_i = min_i_over_sigma**2
     recommended_factor = [
         (target_i / scaled_debye_waller(target_d, *fit))
         for target_d in desired_d_star_sq
@@ -367,23 +368,23 @@ def suggest_minimum_exposure(expts, refls, params):
     recommendations = sorted(recommendations, key=lambda rec: rec[0], reverse=True)
 
     # Print a recommendation to the user.
-    info(u"\nFitted isotropic displacement parameter, B = %.3g Å²", fit[0])
+    info("\nFitted isotropic displacement parameter, B = %.3g Å²", fit[0])
     for target, recommendation in recommendations:
         if recommendation < 1:
             debug(
-                u"\nIt is likely that you can achieve a resolution of %g Å using a "
+                "\nIt is likely that you can achieve a resolution of %g Å using a "
                 "lower exposure (lower transmission and/or shorter exposure time).",
                 target,
             )
         elif recommendation > 1:
             debug(
                 "\nIt is likely that you need a higher exposure (higher transmission "
-                u"and/or longer exposure time to achieve a resolution of %g Å.",
+                "and/or longer exposure time to achieve a resolution of %g Å.",
                 target,
             )
         debug(
-            u"The estimated minimal sufficient exposure (flux × exposure time) to "
-            u"achievea resolution of %.2g Å is %.3g times the exposure used for this "
+            "The estimated minimal sufficient exposure (flux × exposure time) to "
+            "achievea resolution of %.2g Å is %.3g times the exposure used for this "
             "data collection.",
             target,
             recommendation,
@@ -392,12 +393,12 @@ def suggest_minimum_exposure(expts, refls, params):
     summary = "\nRecommendations summarised:\n"
     summary += tabulate(
         recommendations,
-        [u"Resolution (Å)", "Suggested\nexposure factor"],
+        ["Resolution (Å)", "Suggested\nexposure factor"],
         floatfmt=(".3g", ".3g"),
         tablefmt="rst",
     )
     summary += (
-        u"\nExposure is flux × exposure time."
+        "\nExposure is flux × exposure time."
         "\nYou can achieve your desired exposure factor by modifying "
         "transmission and/or exposure time."
     )
@@ -414,8 +415,11 @@ def suggest_minimum_exposure(expts, refls, params):
     )
 
 
-def run(phil=phil_scope, args=None, set_up_logging=False):
-    # type: (scope, Optional[List[str, ...]], bool) -> None
+def run(
+    phil: scope = phil_scope,
+    args: Optional[List[str]] = None,
+    set_up_logging: bool = False,
+) -> None:
     """
     Parse command-line arguments, run the script.
 
@@ -428,7 +432,7 @@ def run(phil=phil_scope, args=None, set_up_logging=False):
         args: Arguments to parse. If None, :data:`sys.argv[1:]` will be used.
         set_up_logging: Choose whether to configure :module:`screen19` logging.
     """
-    usage = "%prog [options] integrated.expt integrated.refl"
+    usage = "%(prog)s [options] integrated.expt integrated.refl"
 
     parser = OptionParser(
         usage=usage,
@@ -446,10 +450,9 @@ def run(phil=phil_scope, args=None, set_up_logging=False):
         log.config(params.verbosity, params.output.log)
 
     if not (params.input.experiments and params.input.reflections):
-        version_information = "screen19.minimum_exposure v%s using %s (%s)" % (
-            __version__,
-            dials_version(),
-            time.strftime("%Y-%m-%d %H:%M:%S"),
+        version_information = (
+            f"screen19.minimum_exposure v{__version__} using {dials_version()} "
+            f"({time.strftime('%Y-%m-%d %H:%M:%S')})"
         )
 
         print(help_message)
@@ -485,7 +488,6 @@ def run(phil=phil_scope, args=None, set_up_logging=False):
     suggest_minimum_exposure(expts, refls, params)
 
 
-def main():
-    # type: () -> None
+def main() -> None:
     """Dispatcher for command-line call."""
     run(set_up_logging=True)
