@@ -1,6 +1,4 @@
-# coding: utf-8
-
-u"""
+"""
 Process screening data obtained at Diamond Light Source Beamline I19.
 
 This program presents the user with recommendations for adjustments to beam
@@ -42,8 +40,6 @@ Examples:
 
 """
 
-from __future__ import absolute_import, division, print_function
-
 import json
 import logging
 import math
@@ -53,10 +49,10 @@ import sys
 import time
 import timeit
 from glob import glob
+from pickle import PickleError
 from typing import Dict, List, Optional, Sequence, Tuple
 
 import procrunner
-from six.moves.cPickle import PickleError
 
 import iotbx.phil
 from libtbx import Auto
@@ -87,7 +83,6 @@ from dxtbx.model.experiment_list import (
     BeamComparison,
     DetectorComparison,
     ExperimentListFactory,
-    ExperimentListTemplateImporter,
     GoniometerComparison,
 )
 
@@ -97,7 +92,7 @@ from screen19.minimum_exposure import suggest_minimum_exposure
 Templates = List[Tuple[str, Tuple[int, int]]]
 
 phil_scope = iotbx.phil.parse(
-    u"""
+    """
     verbosity = 0
         .type = int(value_min=0)
         .caption = 'Verbosity level of log output'
@@ -210,18 +205,17 @@ phil_scope = iotbx.phil.parse(
     process_includes=True,
 )
 
-procrunner_debug = False
-
 logger = logging.getLogger("dials.screen19")
 debug, info, warning = logger.debug, logger.info, logger.warning
 
 
-def _run_integration(scope, experiments_file, reflections_file):
-    # type: (scope, str, str) -> Tuple[ExperimentList, flex.reflection_table]
-    """Run integration programatically, compatible with multiple DIALS versions.
+def _run_integration(
+    phil_scope: scope, experiments_file: str, reflections_file: str
+) -> Tuple[ExperimentList, flex.reflection_table]:
+    """Run integration programmatically, compatible with multiple DIALS versions.
 
     Args:
-        scope: The dials.integrate phil scope
+        phil_scope: The dials.integrate phil scope
         experiments_file: Path to the experiment list file
         reflections_file: Path to the reflection table file
     """
@@ -229,13 +223,13 @@ def _run_integration(scope, experiments_file, reflections_file):
     if hasattr(dials.command_line.integrate, "run_integration"):
         # DIALS 3.1+ interface
         expts, refls, _ = dials.command_line.integrate.run_integration(
-            scope.extract(),
+            phil_scope.extract(),
             ExperimentList.from_file(experiments_file),
             flex.reflection_table.from_file(reflections_file),
         )
     elif hasattr(dials.command_line.integrate, "Script"):
         # Pre-3.1-style programmatic interface
-        expts, refls = dials.command_line.integrate.Script(phil=scope).run(
+        expts, refls = dials.command_line.integrate.Script(phil=phil_scope).run(
             [experiments_file, reflections_file]
         )
     else:
@@ -246,8 +240,11 @@ def _run_integration(scope, experiments_file, reflections_file):
     return expts, refls
 
 
-def overloads_histogram(d_spacings, ticks=None, output="overloads"):
-    # type: (Sequence[float], Optional[Sequence[float]], Optional[str]) -> None
+def overloads_histogram(
+    d_spacings: Sequence[float],
+    ticks: Optional[Sequence[float]] = None,
+    output: Optional[str] = "overloads",
+) -> None:
     """
     Generate a histogram of reflection d-spacings as an image, default is .png.
 
@@ -262,10 +259,10 @@ def overloads_histogram(d_spacings, ticks=None, output="overloads"):
     matplotlib.use("Agg")
     from matplotlib import pyplot as plt
 
-    plt.xlabel(u"d (Å) (inverse scale)")
-    plt.ylabel(u"Number of overloaded reflections")
+    plt.xlabel("d (Å) (inverse scale)")
+    plt.ylabel("Number of overloaded reflections")
     if ticks:
-        plt.xticks([1 / d for d in ticks], ["%g" % d for d in ticks])
+        plt.xticks([1 / d for d in ticks], [f"{d:g}" for d in ticks])
 
     # Matplotlib v3.3.0 includes API change 'nonposy' → 'nonpositive'
     # https://matplotlib.org/api/api_changes.html#log-symlog-scale-base-ticks-and-nonpos-specification
@@ -279,7 +276,7 @@ def overloads_histogram(d_spacings, ticks=None, output="overloads"):
     plt.close()
 
 
-class Screen19(object):
+class Screen19:
     """Encapsulates the screening script."""
 
     def __init__(self):
@@ -291,7 +288,7 @@ class Screen19(object):
         # iotbx.phil.parse.  Confused?  Blame PHIL.
         self.params = phil_scope.fetch(iotbx.phil.parse("")).extract()
 
-    def _quick_import(self, files):  # type: (List[str]) -> bool
+    def _quick_import(self, files: List[str]) -> bool:
         """
         Generate xia2-style templates from file names and attempt a quick import.
 
@@ -318,7 +315,7 @@ class Screen19(object):
             return False
         debug("Attempting quick import...")
         files.sort()
-        templates = {}  # type: Dict[str, List[Optional[List[int]]]]
+        templates: Dict[str, List[Optional[List[int]]]] = {}
         for f in files:
             template, image = screen19.make_template(f)
             if template not in templates:
@@ -332,12 +329,12 @@ class Screen19(object):
             else:
                 templates[template].append([image, image])
         # Return tuple of template and image range for each unique image range
-        templates = [
+        templates: Templates = [
             (t, tuple(r)) for t, ranges in templates.items() for r in ranges
-        ]  # type: Templates
+        ]
         return self._quick_import_templates(templates)
 
-    def _quick_import_templates(self, templates):  # type: (Templates) -> bool
+    def _quick_import_templates(self, templates: Templates) -> bool:
         """
         Take image file templates and frame number ranges and try to run dials.import.
 
@@ -359,7 +356,7 @@ class Screen19(object):
             return False
 
         try:
-            scan_range = templates[0][1]  # type: Tuple[int, int]
+            scan_range: Tuple[int, int] = templates[0][1]
             if not scan_range:
                 raise IndexError
         except IndexError:
@@ -374,7 +371,7 @@ class Screen19(object):
 
         return True
 
-    def _import(self, files):  # type: (List[str]) -> None
+    def _import(self, files: List[str]) -> None:
         """
         Try to run a quick call of dials.import.  Failing that, run a slow call.
 
@@ -419,7 +416,7 @@ class Screen19(object):
                 )
                 try:
                     self.expts = ExperimentList.from_file(files[0])
-                except (IOError, PickleError, ValueError):
+                except (OSError, PickleError, ValueError):
                     pass
                 else:
                     self.params.dials_import.output.experiments = files[0]
@@ -493,7 +490,7 @@ class Screen19(object):
                     scan_tolerance=scan_tolerance,
                     format_kwargs=format_kwargs,
                 )
-            except IOError as e:
+            except OSError as e:
                 warning("%s '%s'", e.strerror, e.filename)
                 sys.exit(1)
 
@@ -507,16 +504,17 @@ class Screen19(object):
         else:
             # Use the template importer.
             if len(self.params.dials_import.input.template) > 0:
-                importer = ExperimentListTemplateImporter(
-                    self.params.dials_import.input.template, format_kwargs=format_kwargs
+                experiments = ExperimentList.from_templates(
+                    self.params.dials_import.input.template,
+                    format_kwargs=format_kwargs,
                 )
                 # Record the imported experiments for use elsewhere.
                 # Quit if there aren't any.
-                self.expts.extend(importer.experiments)
+                self.expts.extend(experiments)
                 if not self.expts:
                     warning(
-                        "No images found matching template %s"
-                        % self.params.dials_import.input.template[0]
+                        "No images found matching template %s",
+                        self.params.dials_import.input.template[0],
                     )
                     sys.exit(1)
 
@@ -526,7 +524,7 @@ class Screen19(object):
         # Extract the experiments and loop through
         self.expts = metadata_updater(self.expts.imagesets())
 
-    def _count_processors(self, nproc=None):  # type: (Optional[int]) -> None
+    def _count_processors(self, nproc: Optional[int] = None) -> None:
         """
         Determine the number of processors and save it as an instance variable.
 
@@ -534,7 +532,7 @@ class Screen19(object):
         given, the number of available processors is returned.
 
         Args:
-            nproc (optional):  Number of processors.
+            nproc:  Number of processors.
         """
         if nproc and nproc is not Auto:
             self.nproc = nproc
@@ -556,7 +554,7 @@ class Screen19(object):
             )
             sys.exit(1)
 
-    def _count_images(self):  # type: () -> int
+    def _count_images(self) -> int:
         """
         Attempt to determine the number of diffraction images.
 
@@ -574,7 +572,7 @@ class Screen19(object):
             warning("Could not determine number of images in dataset.")
             sys.exit(1)
 
-    def _check_intensities(self, mosaicity_correction=True):  # type: (bool) -> None
+    def _check_intensities(self, mosaicity_correction: bool = True) -> None:
         """
         Run xia2.overload and plot a histogram of pixel intensities.
 
@@ -583,17 +581,20 @@ class Screen19(object):
         correction.  See https://github.com/xia2/screen19/wiki#mosaicity-correction
 
         Args:
-            mosaicity_correction (optional):  default is `True`.
+            mosaicity_correction:  default is `True`.
         """
         info("\nTesting pixel intensities...")
-        command = ["xia2.overload", "nproc=%s" % self.nproc, "indexed.expt"]
-        debug("running %s", command)
-        result = procrunner.run(command, print_stdout=False, debug=procrunner_debug)
-        debug("result = %s", screen19.prettyprint_procrunner(result))
-        info("Successfully completed (%.1f sec)", result["runtime"])
+        command = ["xia2.overload", f"nproc={self.nproc}", "indexed.expt"]
 
-        if result["exitcode"] != 0:
-            warning("Failed with exit code %d", result["exitcode"])
+        debug("running %s", command)
+        start = timeit.default_timer()
+        result = procrunner.run(command, print_stdout=False)
+
+        debug("result = %s", screen19.prettyprint_procrunner(result))
+        info("Successfully completed (%.1f sec)", timeit.default_timer() - start)
+
+        if result.returncode:
+            warning("Failed with exit code %d", result.returncode)
             sys.exit(1)
 
         with open("overload.json") as fh:
@@ -618,9 +619,9 @@ class Screen19(object):
                 delta_z = self._oscillation / self._sigma_m / math.sqrt(2)
                 average_to_peak = (
                     math.sqrt(math.pi) * delta_z * math.erf(delta_z)
-                    + math.exp(-(delta_z ** 2))
+                    + math.exp(-(delta_z**2))
                     - 1
-                ) / delta_z ** 2
+                ) / delta_z**2
                 info("Average-to-peak intensity ratio: %f", average_to_peak)
 
         scale = 100 * overload_data["scale_factor"] / average_to_peak
@@ -628,15 +629,15 @@ class Screen19(object):
 
         debug(
             "intensity histogram: { %s }",
-            ", ".join(["%d:%d" % (k, hist[k]) for k in sorted(hist)]),
+            ", ".join([f"{k:d}:{hist[k]:d}" for k in sorted(hist)]),
         )
         max_count = max(hist.keys())
         hist_max = max_count * scale
-        hist_granularity, hist_format = 1, "%.0f"
+        hist_granularity, hist_format = 1, ".0f"
         if hist_max < 50:
-            hist_granularity, hist_format = 2, "%.1f"
+            hist_granularity, hist_format = 2, ".1f"
         if hist_max < 15:
-            hist_granularity, hist_format = 10, "%.1f"
+            hist_granularity, hist_format = 10, ".1f"
         rescaled_hist = {}
         for x in hist.keys():
             rescaled = round(x * scale * hist_granularity)
@@ -647,25 +648,19 @@ class Screen19(object):
             "rescaled histogram: { %s }",
             ", ".join(
                 [
-                    (hist_format + ":%d") % (k / hist_granularity, hist[k])
+                    f"{k / hist_granularity:{hist_format}}:{hist[k]:d}"
                     for k in sorted(hist)
                 ]
             ),
         )
 
-        screen19.plot_intensities(
-            hist, 1 / hist_granularity, procrunner_debug=procrunner_debug
-        )
+        screen19.plot_intensities(hist, 1 / hist_granularity)
 
         linear_response_limit = 100 * self.params.maximum_flux.trusted_range_correction
         marginal_limit = max(70, linear_response_limit)
 
-        text = "".join(
-            (
-                "Strongest pixel (%d counts) " % max_count,
-                "reaches %.1f%% " % hist_max,
-                "of the detector count rate limit",
-            )
+        text = (
+            f"Strongest pixel ({max_count:d} counts) reaches {hist_max:.1f}% of the detector count rate limit",
         )
         if hist_max > 100:
             warning("Warning: %s!", text)
@@ -681,28 +676,25 @@ class Screen19(object):
                 "limits of the detector.\n"
                 "         The built-in detector count rate correction cannot "
                 "adjust for this.\n"
-                "         You should aim for count rates below {:.0%} of the "
-                "detector limit.".format(
-                    self.params.maximum_flux.trusted_range_correction
-                )
+                "         You should aim for count rates below "
+                f"{self.params.maximum_flux.trusted_range_correction:.0%} of the "
+                "detector limit."
             )
         elif hist_max > marginal_limit:
             warning(
                 "Warning: The photon incidence rate is well outside the "
-                "linear response region of the detector (<{:.0%}).\n"
+                "linear response region of the detector "
+                f"(<{self.params.maximum_flux.trusted_range_correction:.0%}).\n"
                 "    The built-in detector count rate correction may not be "
-                "able to adjust for this.".format(
-                    self.params.maximum_flux.trusted_range_correction
-                )
+                "able to adjust for this."
             )
         elif hist_max > linear_response_limit:
             info(
                 "The photon incidence rate is outside the linear response "
-                "region of the detector (<{:.0%}).\n"
+                "region of the detector "
+                f"(<{self.params.maximum_flux.trusted_range_correction:.0%}).\n"
                 "    The built-in detector count rate correction may be able "
-                "to adjust for this.".format(
-                    self.params.maximum_flux.trusted_range_correction
-                )
+                "to adjust for this."
             )
         if not mosaicity_correction:
             warning(
@@ -714,13 +706,12 @@ class Screen19(object):
 
         info("Total sum of counts in dataset: %d", count_sum)
 
-    def _find_spots(self, args=None):  # type: (Optional[List[str]]) -> None
+    def _find_spots(self, args: Optional[List[str]] = None) -> None:
         """
         Call `dials.find_spots` on the imported experiment list.
 
         Args:
-            args (optional):  List of any additional PHIL parameters to be used by
-                              dials.import.
+            args:  List of any additional PHIL parameters to be used by dials.import.
         """
         info("\nFinding spots...")
 
@@ -750,7 +741,7 @@ class Screen19(object):
             timeit.default_timer() - dials_start,
         )
 
-    def _index(self):  # type: () -> bool
+    def _index(self) -> bool:
         """
         Call `dials.index` on the output of spot finding.
 
@@ -834,7 +825,7 @@ class Screen19(object):
         # Report the indexing successful.
         return True
 
-    def _wilson_calculation(self):  # type: () -> None
+    def _wilson_calculation(self) -> None:
         """
         Run `screen19.minimum_exposure` on an experiment list and reflection table.
 
@@ -849,7 +840,7 @@ class Screen19(object):
 
         info("Successfully completed (%.1f sec)", timeit.default_timer() - dials_start)
 
-    def _refine(self):  # type: () -> None
+    def _refine(self) -> None:
         """
         Run `dials.refine` on the results of indexing.
         """
@@ -866,7 +857,7 @@ class Screen19(object):
 
         info("Successfully refined (%.1f sec)", timeit.default_timer() - dials_start)
 
-    def _create_profile_model(self):  # type: () -> bool
+    def _create_profile_model(self) -> bool:
         """
         Run `dials.create_profile_model` on indexed reflections.
 
@@ -882,27 +873,30 @@ class Screen19(object):
             "dials.create_profile_model",
             self.params.dials_index.output.experiments,
             self.params.dials_index.output.reflections,
-            "output = %s" % self.params.dials_index.output.experiments,
+            f"output = {self.params.dials_index.output.experiments}",
         ]
-        result = procrunner.run(command, print_stdout=False, debug=procrunner_debug)
+
+        start = timeit.default_timer()
+        result = procrunner.run(command, print_stdout=False)
+
         debug("result = %s", screen19.prettyprint_procrunner(result))
         self._sigma_m = None
-        if result["exitcode"] == 0:
+        if result.returncode == 0:
             db = ExperimentList.from_file(self.params.dials_index.output.experiments)[0]
             self._oscillation = db.imageset.get_scan().get_oscillation()[1]
             self._sigma_m = db.profile.sigma_m()
             info(
-                u"%d images, %s° oscillation, σ_m=%.3f°",
+                "%d images, %s° oscillation, σ_m=%.3f°",
                 db.imageset.get_scan().get_num_images(),
                 str(self._oscillation),
                 self._sigma_m,
             )
-            info("Successfully completed (%.1f sec)", result["runtime"])
+            info("Successfully completed (%.1f sec)", timeit.default_timer() - start)
             return True
-        warning("Failed with exit code %d", result["exitcode"])
+        warning("Failed with exit code %d", result.returncode)
         return False
 
-    def _integrate(self):  # type: () -> None
+    def _integrate(self) -> None:
         """Run `dials.integrate` to integrate reflection intensities."""
         dials_start = timeit.default_timer()
         info("\nIntegrating...")
@@ -941,8 +935,9 @@ class Screen19(object):
     # This is a hacky check but should work for as long as DIALS 2.0 is supported.
     if version.dials_version() < "DIALS 2.1":
 
-        def _refine_bravais(self, experiments, reflections):
-            # type: (ExperimentList, flex.reflection_table) -> None
+        def _refine_bravais(
+            self, experiments: ExperimentList, reflections: flex.reflection_table
+        ) -> None:
             """
             Run `dials.refine_bravais_settings` on an experiments and reflections.
 
@@ -952,12 +947,15 @@ class Screen19(object):
             """
             info("\nRefining Bravais settings...")
             command = ["dials.refine_bravais_settings", experiments, reflections]
-            result = procrunner.run(command, print_stdout=False, debug=procrunner_debug)
+
+            start = timeit.default_timer()
+            result = procrunner.run(command, print_stdout=False)
+
             debug("result = %s", screen19.prettyprint_procrunner(result))
-            if result["exitcode"] == 0:
+            if result.returncode == 0:
                 m = re.search(
                     r"[-+]{3,}\n[^\n]*\n[-+|]{3,}\n(.*\n)*[-+]{3,}",
-                    result["stdout"].decode("utf-8"),
+                    result.stdout.decode("utf-8"),
                 )
                 if m:
                     info(m.group(0))
@@ -966,14 +964,16 @@ class Screen19(object):
                         "Could not interpret dials.refine_bravais_settings output, "
                         "please check dials.refine_bravais_settings.log"
                     )
-                info("Successfully completed (%.1f sec)", result["runtime"])
+                info(
+                    "Successfully completed (%.1f sec)", timeit.default_timer() - start
+                )
             else:
-                warning("Failed with exit code %d", result["exitcode"])
+                warning("Failed with exit code %d", result.returncode)
                 sys.exit(1)
 
     else:
 
-        def _refine_bravais(self):  # type: () -> None
+        def _refine_bravais(self) -> None:
             """Run `dials.refine_bravais_settings` to determine the space group."""
             dials_start = timeit.default_timer()
             info("\nRefining Bravais settings...")
@@ -981,9 +981,18 @@ class Screen19(object):
             self.refls = eliminate_sys_absent(self.expts, self.refls)
             map_to_primitive(self.expts, self.refls)
 
+            # We can only refine against reflections with non-zero variance in
+            # observed centroid position.
+            x_valid, y_valid, z_valid = (
+                part > 0 for part in self.refls["xyzobs.mm.variance"].parts()
+            )
+            nonzero_variance = x_valid | y_valid | z_valid
+
             try:
                 refined_settings = refined_settings_from_refined_triclinic(
-                    self.expts, self.refls, self.params.dials_refine_bravais
+                    self.expts,
+                    self.refls.select(nonzero_variance),
+                    self.params.dials_refine_bravais,
                 )
             except RuntimeError as e:
                 warning("dials.refine_bravais_settings failed.\nGiving up.")
@@ -1005,8 +1014,9 @@ class Screen19(object):
                 timeit.default_timer() - dials_start,
             )
 
-    def _report(self, experiments, reflections):
-        # type: (ExperimentList, flex.reflection_table) -> None
+    def _report(
+        self, experiments: ExperimentList, reflections: flex.reflection_table
+    ) -> None:
         """
         Run `dials.report` on an experiment list and reflection table.
 
@@ -1016,10 +1026,13 @@ class Screen19(object):
         """
         info("\nCreating report...")
         command = ["dials.report", experiments, reflections]
-        result = procrunner.run(command, print_stdout=False, debug=procrunner_debug)
+
+        start = timeit.default_timer()
+        result = procrunner.run(command, print_stdout=False)
+
         debug("result = %s", screen19.prettyprint_procrunner(result))
-        if result["exitcode"] == 0:
-            info("Successfully completed (%.1f sec)", result["runtime"])
+        if result.returncode == 0:
+            info("Successfully completed (%.1f sec)", timeit.default_timer() - start)
         #     if sys.stdout.isatty():
         #       info("Trying to start browser")
         #       try:
@@ -1030,11 +1043,15 @@ class Screen19(object):
         #       except Exception as e:
         #         debug("Could not open browser\n%s", str(e))
         else:
-            warning("Failed with exit code %d", result["exitcode"])
+            warning("Failed with exit code %d", result.returncode)
             sys.exit(1)
 
-    def run(self, args=None, phil=phil_scope, set_up_logging=False):
-        # type: (Optional[List[str]], scope, bool) -> None
+    def run(
+        self,
+        args: Optional[List[str]] = None,
+        phil: scope = phil_scope,
+        set_up_logging: bool = False,
+    ) -> None:
         """
         TODO: Docstring.
 
@@ -1046,7 +1063,7 @@ class Screen19(object):
         Returns:
 
         """
-        usage = "%prog [options] image_directory | image_files.cbf | imported.expt"
+        usage = "%(prog)s [options] image_directory | image_files.cbf | imported.expt"
 
         parser = OptionParser(
             usage=usage, epilog=__doc__, phil=phil, check_format=False
@@ -1056,10 +1073,10 @@ class Screen19(object):
             args=args, show_diff_phil=True, return_unhandled=True, quick_parse=True
         )
 
-        version_information = "screen19 v%s using %s (%s)" % (
-            screen19.__version__,
-            dials.util.version.dials_version(),
-            time.strftime("%Y-%m-%d %H:%M:%S"),
+        version_information = (
+            f"screen19 v{screen19.__version__} "
+            f"using {dials.util.version.dials_version()} "
+            f"({time.strftime('%Y-%m-%d %H:%M:%S')})"
         )
 
         start = timeit.default_timer()
@@ -1170,6 +1187,6 @@ class Screen19(object):
         info("screen19 successfully completed (%.1f sec).", runtime)
 
 
-def main():  # type: () -> None
+def main() -> None:
     """Dispatcher for command-line call."""
     Screen19().run(set_up_logging=True)
